package bytecrafters.inspector;

import bytecrafters.reflect.Classes;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.*;
import javax.swing.border.Border;
import java.awt.*;
import java.net.URL;

/**
 * Author: Rhett
 * Date: Feb 17, 2003
 * Time: 1:16:41 AM
 */
public abstract class Inspector extends DefaultMutableTreeNode  {
    private String label;
    private boolean loaded = false;
    private static Image titleBarImage;
    private static ImageIcon objectOpenIcon;
    private static ImageIcon objectClosedIcon;
    private static ImageIcon primitiveIcon;

    static {
        titleBarImage = loadIcon( "inspector").getImage();
        objectOpenIcon = loadIcon( "object-node-open");
        objectClosedIcon = loadIcon( "object-node-closed");
        primitiveIcon = loadIcon( "primitive-node");

    }

    private static ImageIcon loadIcon( String name ){
        URL url = Inspector.class.getClassLoader().getResource( "images/inspector/" + name + ".gif" );
        return new ImageIcon( url );
    }

    public static void inspect( Object object, boolean isPrimitive ){
        JFrame frame = new JFrame();
        Class type = isPrimitive ? Classes.getPrimitiveType(object) : object.getClass();
        JTree tree = new JTree( createInspector( Classes.getPrintableName( type ), type, object ) );
        tree.setRowHeight( objectOpenIcon.getIconHeight() );

        DefaultTreeCellRenderer renderer = (DefaultTreeCellRenderer)tree.getCellRenderer();
        renderer.setOpenIcon(objectOpenIcon);
        renderer.setClosedIcon(objectOpenIcon);
        renderer.setLeafIcon(primitiveIcon);

        JScrollPane treeView = new JScrollPane(tree);
        Border etch = BorderFactory.createEtchedBorder();
        Border margin = BorderFactory.createLineBorder(frame.getBackground(), 5);
        Border outer = BorderFactory.createCompoundBorder( margin, etch );
        Border whiteLine = BorderFactory.createLineBorder( Color.WHITE, 1  );
        Border blackLine = BorderFactory.createLineBorder(Color.BLACK, 1 );
        Border inner = BorderFactory.createCompoundBorder( whiteLine, blackLine );
        Border border = BorderFactory.createCompoundBorder( outer, inner );
        treeView.setBorder( border );
        frame.getContentPane().add( treeView );
        frame.setTitle( "Scrappy Inspector" );
        frame.setIconImage( titleBarImage );
        frame.setDefaultCloseOperation( frame.DISPOSE_ON_CLOSE );
        frame.setSize( 640, 480 );
        frame.setLocationRelativeTo( null );
        frame.setVisible(true);
    }

    public Inspector( String label, Object userObject, boolean allowsChildren) {
        super(userObject, allowsChildren);
        this.label = label;
    }

    public String toString(){
        return label;
    }

    public int getChildCount() {
        if( !loaded && allowsChildren ) {
            loaded = true;
            loadChildren();
        }

        return super.getChildCount();
    }

    protected abstract void loadChildren();

    protected static MutableTreeNode createInspector( String label, Class type, Object value ){
        label = label + " = " + Classes.toString( value );

        if( type.isArray() )
            return new ArrayInspector( label, value );
        else if( Object.class.isAssignableFrom( type ) )
            return new ObjectInspector( label, value );
        else
            return new DefaultMutableTreeNode(label, false );
    }
}
