package bytecrafters.inspector;

import bytecrafters.reflect.Classes;
import bytecrafters.reflect.Arrays;

import java.lang.reflect.Field;

/**
 * Author: Rhett
 * Date: Feb 25, 2003
 * Time: 12:13:02 AM
 */
public class ObjectInspector extends Inspector {

    public ObjectInspector( String label, Object value ){
        super( label, value, value != null );
    }

    protected void loadChildren() {
       Field[] fields = userObject.getClass().getDeclaredFields();

        for( int i = 0; i < fields.length; i++ ){
            Field field = fields[i];
            String childLabel = field.getName() + ": " + Classes.getPrintableName( field.getType() );
            add( createInspector( childLabel, field.getType(), getFieldValue( field ) ) );
        }
    }

    private Object getFieldValue( Field field ){
        try{
            field.setAccessible( true );
            return field.get(userObject);
        }
        catch( IllegalAccessException ex ){
            return null;
        }
    }

    protected String getArrayFieldLabel( Field field, Object value ){
        String typeString = Classes.getPrintableName( field.getType() );
        String valueString = Arrays.toString( value );
        return field.getName() + ": " + typeString + " = " + valueString;
    }

    protected String getObjectFieldLabel( Field field, Object value ){
        String valueString = value == null ? "null" : value.toString();
        return getLabel( field, "{" + valueString + "}" );
    }

    protected String getLabel( Field field, String valueString ){
        return field.getName() + ": " + field.getType() + " = " + valueString;
    }
}