package bytecrafters.scrappy;


import net.sf.pizzacompiler.compiler.*;

import javax.swing.*;
import java.awt.*;
import java.io.*;
import java.util.StringTokenizer;
import java.util.HashMap;

/**
 * Author: Rhett
 * Date: Feb 17, 2003
 * Time: 10:28:41 AM
 */
public abstract class ExecuteIt {
    private static final String SCRAP_CLASS = "scrap";
    private IClassContext classContext;
    private ILogger logger;

    public ExecuteIt(IClassContext classContext, ILogger logger ) {
        this.classContext = classContext;
        this.logger = logger;
    }

// ###########################################################################
//                        TEMPLATE METHOD FOR EXECUTION
// ###########################################################################

    public void executeIt( String importedPackages, JTextArea scrapArea ) {
        scrapArea.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        String scrapCode = scrapArea.getSelectedText();

        if( scrapCode == null )
            scrapCode = scrapArea.getText().trim();
        else
            scrapCode = scrapCode.trim();

        StringBuffer source = new StringBuffer();
        int importLines = addImports( importedPackages, source );
        int nonScrapLines = 1 + importLines;
        source.append( "public class " ).append( SCRAP_CLASS ).append( " implements " ).append( IScrap.class.getName() );
        source.append( "{" );
        source.append( "public void execute( " ).append( ScrapResult.class.getName() ).append( " result ) throws Exception {\r\n" );
        source.append(    getMethodBody(scrapCode) );
        source.append( "\r\n}" );
        source.append( "}" );

        try {
            IScrap scrap = createScrap( SCRAP_CLASS, source.toString() );
            ScrapResult result = new ScrapResult();
            scrap.execute(result);
            displayResult(scrapArea, result);
        }
        catch (CompileFailedException ex) {
            String compilerMessages = ex.getMessage();
            if (compilerMessages.indexOf(": not a statement") > 0) // then assume they tried to doIt on an expression
                displayCompileError(scrapArea, "Code executed with 'Do It' must end in a statement\n");
            else if (compilerMessages.endsWith( "illegal start of expression\r\n(source unavailable)\r\n1 error\r\n") ) // the nassume they tried to printIt on a statement
                displayCompileError(scrapArea, "Code executed with 'Print It' or 'Inspect It' must end in an expression\n");
            else
                displayCompileError(scrapArea, filterCompilerOutput(compilerMessages, nonScrapLines ));
        } catch (Exception ex) {
            displayException(scrapArea, ex, nonScrapLines);
        } finally {
            scrapArea.setCursor(Cursor.getDefaultCursor());
        }
    }

//                  HOOK METHODS
    protected abstract String getMethodBody(String scrapCode);

    protected abstract void displayResult(JTextArea scrapArea, ScrapResult result);


    protected String methodThatCapturesResult(String scrapCode) {
        StringBuffer methodBody = new StringBuffer();
        StringTokenizer lines = new StringTokenizer(scrapCode, "\r\n");
        int lineCount = lines.countTokens();

        for (int i = 0; i < lineCount - 1; i++)
            methodBody.append(lines.nextToken()).append("\n");

        String lastLine = lines.nextToken().trim();

        if (lastLine.endsWith(";"))
            lastLine = lastLine.substring(0, lastLine.length() - 1);

        methodBody.append("result.setValue( ").append(lastLine).append(" );");
        return methodBody.toString();
    }
    public IScrap createScrap(String className, String source) throws CompileFailedException {
        ByteArrayOutputStream compilerMessageBytes =  new ByteArrayOutputStream();

        try {
            Main.init();
            ClassLoader contextLoader = classContext.getContextClassLoader();
            ByteArrayCompilerOutput output = new ByteArrayCompilerOutput();
            HashMap classMap = new HashMap();
            classMap.put(className, source);
            SourceReader reader = new MapSourceReader(classMap);
            PrintStream sysOut = new PrintStream(compilerMessageBytes);
            Main.compile(new String[]{className}, reader, output, sysOut);
            ClassLoader scrapLoader = new ByteArrayClassLoader(contextLoader, output.getBytecode());
            return (IScrap)scrapLoader.loadClass(className).newInstance();
        }
        catch( Throwable throwable ){
            String throwableString = throwable.toString();
            if( throwableString == null || !throwableString.equals( "java.lang.ClassNotFoundException: scrap"))
                logger.log( throwable );
            throw new CompileFailedException( new String(compilerMessageBytes.toByteArray()));
        }
    }

    private String filterCompilerOutput( String compilerMessages, int nonScrapLines ) {
        StringTokenizer lines = new StringTokenizer(compilerMessages, "\n");
        StringBuffer filteredLines = new StringBuffer(compilerMessages.length());
        String scrapLineReport = SCRAP_CLASS + ":";
        int lineNumberBegin = scrapLineReport.length();

        while (lines.hasMoreTokens()) {
            String line = lines.nextToken().trim();

            if (line.startsWith( scrapLineReport) ) {
                int nextColonIndex = line.indexOf( ':', lineNumberBegin );
                String compilerLineNumber = line.substring(lineNumberBegin, nextColonIndex);
                int correctLineCount = Integer.parseInt(compilerLineNumber) - nonScrapLines;

                if( correctLineCount <= 0 )
                    appendCompilerOutputLine(filteredLines, "import line " + line.substring(nextColonIndex));
                else
                    appendCompilerOutputLine(filteredLines, "scrap line " + correctLineCount + line.substring(nextColonIndex));

            } else if(!line.equals( "(source unavailable)") )
                appendCompilerOutputLine(filteredLines, line);
        }

        return filteredLines.toString();
    }

    private void appendCompilerOutputLine( StringBuffer filteredLines, String line ){
        String scrapArtifact = " in method execute(bytecrafters.scrappy.ScrapResult)";

        if( line.endsWith( scrapArtifact ) )
            line = line.substring(0, line.length() - scrapArtifact.length() );

        filteredLines.append( line ).append( "\n" );
    }

    private int addImports( String importedPackages, StringBuffer source ){
        if( importedPackages == null )
            return 0;

        StringTokenizer tokens = new StringTokenizer( importedPackages, ";" );
        int importCount = tokens.countTokens();
        while( tokens.hasMoreTokens() ){
            String importedPackage = tokens.nextToken().trim();

            if( importedPackage.length() != 0 ) {
                if( !importedPackage.startsWith( "import ") )
                    source.append( "import " );

                source.append( importedPackage ).append( ";\r\n" );
            }
        }

        return importCount;
    }



// ###########################################################################
//                        UI AND DISPLAY
// ###########################################################################

    protected void clearSelection(JTextArea scrapArea) {
        scrapArea.setSelectionStart(scrapArea.getSelectionEnd());
    }

    protected void displayString(JTextArea scrapArea, String result) {
        boolean hasSelection = scrapArea.getSelectedText() != null;
        int insertPoint = hasSelection ? scrapArea.getSelectionEnd() : scrapArea.getText().length();
        scrapArea.insert(" " + result, insertPoint);
        scrapArea.setSelectionStart(insertPoint + 1);
        scrapArea.setSelectionEnd(insertPoint + 1 + result.length());
    }

    private void displayCompileError(JTextArea scrapArea, String error) {
        displayString(scrapArea, "\n/*\nCOMPILE FAILED:\n" + error + "*/");
    }

    private void displayException(JTextArea scrapPane, Exception ex, int nonScrapLines ) {
        displayString(scrapPane, printStackTrace( ex, nonScrapLines  ) );
    }

    private String printStackTrace( Exception ex, int nonScrapLines ){
        StringBuffer buf = new StringBuffer();
        buf.append( ex.toString() ).append( "\n" );
        StackTraceElement[] trace = ex.getStackTrace();

        for( int i = 0; i < trace.length; i++ ) {
            String traceClass = trace[i].getClassName();

            if( traceClass.startsWith( "bytecrafters") )
                break;

            buf.append("\t");

            if( traceClass.equals( SCRAP_CLASS ) )
                buf.append( "scrap line " + (trace[i].getLineNumber() - nonScrapLines) );
            else
                buf.append( trace[i].toString() );

            buf.append( "\n" );
        }

        return buf.toString();
    }

}
