package bytecrafters.scrappy;

import bytecrafters.ui.About;
import bytecrafters.ui.Help;
import bytecrafters.reflect.Classes;

import java.awt.event.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;

public class Scrappy {
    private JTextField imports = new JTextField();
    private JTabbedPane scrapTabs = new JTabbedPane();
    private JPopupMenu popupMenu = new JPopupMenu();
    private JMenuItem doItMenuItem = new JMenuItem("Do It");
    private DoIt doIt;
    private JMenuItem printItMenuItem = new JMenuItem("Print It");
    private PrintIt printIt;
    private JMenuItem inspectItMenuItem = new JMenuItem("Inspect It");
    private InspectIt inspectIt;
    private JMenuItem newTabMenuItem = new JMenuItem("New Tab");
    private JMenuItem renameTabMenuItem = new JMenuItem("Rename Tab");
    private JMenuItem deleteTabMenuItem = new JMenuItem("Delete Tab");
    private MouseListener mouseListener = new MouseAdapter() {
        public void mouseReleased(MouseEvent evt) {
            mouseReleasedEvent(evt);
        }
    };

    // ########################################################################
    //                          INITIALIZATION
    // ########################################################################

    public Scrappy( IClassContext classContext, ILogger logger ) {
        doIt = new DoIt( classContext, logger );
        printIt = new PrintIt( classContext, logger );
        inspectIt = new InspectIt( classContext, logger );

        scrapTabs.addChangeListener( new ChangeListener(){
            public void stateChanged(ChangeEvent e) {
                JComponent scrapArea = getSelectedScrapArea();
                if( scrapArea != null ) scrapArea.requestFocus();
            }
        });

        scrapTabs.setToolTipText( "Right-click for commands and help" );
        scrapTabs.addMouseListener(mouseListener);

        doItMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D, ActionEvent.CTRL_MASK));
        doItMenuItem.addActionListener( new ActionListener(){
            public void actionPerformed( ActionEvent e ){
                doIt.executeIt( imports.getText(), getSelectedScrapArea() );
            }
        } );

        printItMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P, ActionEvent.CTRL_MASK));
        printItMenuItem.addActionListener( new ActionListener(){
            public void actionPerformed( ActionEvent e ){
                printIt.executeIt( imports.getText(), getSelectedScrapArea() );
            }
        } );

        inspectItMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_I, ActionEvent.CTRL_MASK));
        inspectItMenuItem.addActionListener(new ActionListener(){
            public void actionPerformed( ActionEvent e ){
                inspectIt.executeIt( imports.getText(), getSelectedScrapArea() );
            }
        } );

        newTabMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, ActionEvent.CTRL_MASK));
        newTabMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                newScrapTab();
            }
        });

        renameTabMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R, ActionEvent.CTRL_MASK));
        renameTabMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                renameScrapTab();
            }
        });

        deleteTabMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, ActionEvent.CTRL_MASK));
        deleteTabMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                deleteScrapTab();
            }
        });

        JMenuItem helpMenuItem = new JMenuItem("Help");
        helpMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                new Help( Classes.getName( Scrappy.class ) + " Help", "help/index.html" );
            }
        });

        JMenuItem aboutMenuItem = new JMenuItem("About");
        aboutMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                About about = new About( "images/about.gif" );
                about.setVisible(true);
            }
        });

        popupMenu.add(doItMenuItem);
        popupMenu.add(printItMenuItem);
        popupMenu.add(inspectItMenuItem);
        popupMenu.addSeparator();
        popupMenu.add(newTabMenuItem);
        popupMenu.add(renameTabMenuItem);
        popupMenu.add(deleteTabMenuItem);
        popupMenu.addSeparator();
        popupMenu.add(helpMenuItem);
        popupMenu.add(aboutMenuItem);

        imports.setToolTipText("Ex: com.foo.*;com.bar.*");
    }

    // ########################################################################
    //                          TAB MANAGEMENT
    // ########################################################################

    public String getImports(){
        return imports.getText();
    }

    public void setImports( String text ){
        imports.setText( text );
    }

    public int getTabCount() {
        return scrapTabs.getTabCount();
    }

    public String getTabName( int index ){
        return scrapTabs.getTitleAt( index) ;
    }

    public JTextArea getScrapArea(int index) {
        JScrollPane scrollPane = (JScrollPane) scrapTabs.getComponent(index);
        return (JTextArea) scrollPane.getViewport().getView();
    }

    JTextArea getSelectedScrapArea() {
        int selectedIndex = scrapTabs.getSelectedIndex();
        return selectedIndex == -1 ? null : getScrapArea( selectedIndex );
    }

    private void newScrapTab(){
        String tabName = JOptionPane.showInputDialog(  scrapTabs, "Tab name:");
        if( tabName != null )
            createScrapTab( tabName, "" );
    }

    public void createDefaultScrapTab(){
        createScrapTab( "Scrap", "" );
    }

    public void createScrapTab( String name, String text) {
        JTextArea scrapArea = new JTextArea(text);
        scrapArea.setTabSize(2);
        scrapArea.addMouseListener(mouseListener);
        scrapArea.addKeyListener(new KeyAdapter() {
            public void keyPressed(KeyEvent evt ) {
                    keyPressedEvent( evt );
            }
        });
        JScrollPane scroller = new JScrollPane(scrapArea);
        scroller.addMouseListener(mouseListener);
        scrapTabs.addTab( name, scroller);
        scrapTabs.setSelectedIndex(getTabCount() - 1);
        scrapArea.requestFocus();
    }

    void deleteScrapTab() {
        scrapTabs.remove(scrapTabs.getSelectedIndex());
    }

    void renameScrapTab(){
        String tabName = JOptionPane.showInputDialog(  scrapTabs, "Tab name:");
        if( tabName != null )
            scrapTabs.setTitleAt( scrapTabs.getSelectedIndex(), tabName );
    }


    // ########################################################################
    //                          UI
    // ########################################################################

    public JComponent getUI() {
        JPanel panel = new JPanel( new BorderLayout( 1, 1) );
        JPanel importsPanel = new JPanel( new BorderLayout( 1, 1) );
        importsPanel.add( new JLabel( "<html><b>imports:</b></html>" ), BorderLayout.WEST );
        importsPanel.add( imports, BorderLayout.CENTER  );
        //importsPanel.add( new JSeparator( JSeparator.HORIZONTAL ), BorderLayout.SOUTH );
        panel.add( importsPanel, BorderLayout.NORTH );
        panel.add( scrapTabs, BorderLayout.CENTER );
        panel.setBorder( BorderFactory.createLineBorder(panel.getBackground(), 2) );
        return panel;
    }

    void refreshMenuEnablement() {
        boolean hasTabs = getTabCount() > 0;
        boolean hasCode = hasTabs && getSelectedScrapArea().getText() != null;
        doItMenuItem.setEnabled(hasCode);
        printItMenuItem.setEnabled(hasCode);
        inspectItMenuItem.setEnabled(hasCode);
        newTabMenuItem.setEnabled(true);
        deleteTabMenuItem.setEnabled(hasTabs);
    }

    private void keyPressedEvent( KeyEvent evt ) {
        if (evt.isControlDown()) {
            refreshMenuEnablement();

            // for some reason the accelerators are not working...
            switch( evt.getKeyCode() ){
                case KeyEvent.VK_D:
                    if( doItMenuItem.isEnabled() )
                        doIt.executeIt( imports.getText(), getSelectedScrapArea() );
                    break;
                case KeyEvent.VK_P:
                    if( printItMenuItem.isEnabled() )
                        printIt.executeIt(imports.getText(), getSelectedScrapArea() );
                    break;
                case KeyEvent.VK_I:
                    if( inspectItMenuItem.isEnabled() )
                        inspectIt.executeIt(imports.getText(), getSelectedScrapArea() );
                    break;
                case KeyEvent.VK_N:
                    if( newTabMenuItem.isEnabled() )
                        newScrapTab();
                    break;
                case KeyEvent.VK_R:
                    if( renameTabMenuItem.isEnabled() )
                        newScrapTab();
                    break;
                case KeyEvent.VK_DELETE:
                    if( deleteTabMenuItem.isEnabled() )
                        deleteScrapTab();
                    break;
            }
        }
    }

    private void mouseReleasedEvent(MouseEvent evt) {
        refreshMenuEnablement();
        if (evt.isPopupTrigger())
            popupMenu.show(evt.getComponent(), evt.getX(), evt.getY());
        else if (popupMenu.isShowing())
            popupMenu.setVisible(false);
    }
}
