package bytecrafters.scrappy.intellij;

import com.intellij.openapi.components.*;
import com.intellij.openapi.wm.*;
import com.intellij.openapi.project.*;
import com.intellij.openapi.util.JDOMExternalizable;
import com.intellij.openapi.util.WriteExternalException;
import com.intellij.openapi.util.InvalidDataException;
import org.jdom.Element;

import javax.swing.*;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.net.URL;

import bytecrafters.scrappy.Scrappy;
import bytecrafters.reflect.Classes;

/**
 * Author: Rhett
 * Date: Feb 16, 2003
 * Time: 1:01:45 PM
 */
public class Plugin implements ProjectComponent, JDOMExternalizable {
    private Project project;
    private Scrappy scrappy;

    public Plugin(Project project) {
        this.project = project;
        this.scrappy = new Scrappy( new IntellijClassContext( project ), new IntellijLogger( Scrappy.class ) );
    }

    public void initComponent() {
        if (scrappy.getTabCount() == 0)
            scrappy.createDefaultScrapTab();
    }

    public void disposeComponent() {
    }

    public void projectOpened() {
        JComponent ui = scrappy.getUI();
        ToolWindow window = ToolWindowManager.getInstance(project).registerToolWindow( getComponentName(), ui, ToolWindowAnchor.BOTTOM);
        URL url = getClass().getClassLoader().getResource( "images/scrappy.gif" );
        window.setIcon(new ImageIcon( url ));
    }

    public void projectClosed() {
    }

    public String getComponentName() {
        return Classes.getName( Scrappy.class );
    }

    public void writeExternal(Element root) throws WriteExternalException {
        for (int i = 0; i < scrappy.getTabCount(); i++) {
            Element scrapTabElement = new Element("scrapTab");
            scrapTabElement.setAttribute("name", scrappy.getTabName(i));
            String tabText = scrappy.getScrapArea(i).getText();
            StringTokenizer lines = new StringTokenizer( tabText, "\n");
            StringBuffer external = new StringBuffer();

             //IDEA does not persist line breaks, I hope the ` character does not show
             //up in anyone's code
            while( lines.hasMoreTokens() )
                external.append( lines.nextToken() ).append( "`");

            scrapTabElement.setText( external.toString() );
            root.addContent(scrapTabElement);
        }

        String imports = scrappy.getImports();

        if( imports != null ) {
            Element importsNode = new Element( "imports" );
            importsNode.setText( imports );
            root.addContent( importsNode );
        }
    }

    public void readExternal(Element element) throws InvalidDataException {
        for (Iterator i = element.getChildren("scrapTab").iterator(); i.hasNext();) {
            Element scrapElement = (Element) i.next();
            String name = scrapElement.getAttributeValue("name");
            StringTokenizer lines = new StringTokenizer( scrapElement.getText(), "`");
            StringBuffer tabText = new StringBuffer();

            while( lines.hasMoreTokens() )
                tabText.append( lines.nextToken() ).append( "\n");

            scrappy.createScrapTab(name, tabText.toString() );
        }

        Element importsNode = element.getChild( "imports" );

        if( importsNode != null )
            scrappy.setImports( importsNode.getText() );
    }
}
